;--------------------------------------------------------------------------
;    fdc_detect.s - generic FDC driver for WD279x (Z80)
;
;    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
;
;    Thies program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    driver_FDC is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with driver_FDC; if not, write to the Free Software
;    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;--------------------------------------------------------------------------



;
; Pri FDC_SPUPPORT_TWO_CONTROLLERS = 1 budou podporovany 2 radice ve
; standardu WD279x. Mechaniky 0 - 3 jsou umisteny na realnem FDC (porty 0xd8
; - 0xdf ).
; Mechaniky 4 -7 jsou emulovany Unikartou (switchem unikarty posunute porty
; 0x58 - 0x5f).
;
FDC_SPUPPORT_TWO_CONTROLLERS = 1


; port stopy
__FDC_TRACK = 0xd9


__FDC_TRACK_TEST_VALUE	= 0xa4	; kvuli unikarte volime jinou testovaci hodnotu, nez se pouziva v ROM



; _delay - modifikovatelna cekaci smycka (volano z ASM)
;
; Vstup:
;	reg. DE * 8 => priblizny cas v us
;
; Poskodi reg.: DE, AF
;
__delay:
    1$:
    dec de
    ld a,d
    or e
    jr nz, 1$
    ret


.if FDC_SPUPPORT_TWO_CONTROLLERS
;
; fdc_detect_unicard - test zda je pritomen emulovany FDC na portech 0x58 - 0x5f
;
; ret_val:
;	-1: nepritomen
;	 0: pritomen
;
_fdc_detect_unicard::
	ld c,#__FDC_TRACK&0x7f
	jr _fdc_detect_test
.endif

;
; fdc_detect - test zda je pritomen FDC
;
; ret_val:
;	-1: nepritomen
;	 0: pritomen
;
_fdc_detect::
    ld c,#__FDC_TRACK

_fdc_detect_test:
    xor a
    ld l,a
    
    ld a,#__FDC_TRACK_TEST_VALUE
    ld b,a
    
    out (c),a
    ld de,#0x000f
    call __delay
    in a,(c)
    cp b
    ret z
    dec l
    ret
