/*
 *    main.c - generic FDC driver for WD279x (Z80)
 *
 *    Copyright (c) 2012 by Michal Hucik <http://www.ordoz.com>
 *
 *    Thies program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    driver_FDC is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with driver_FDC; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>

#include "video.h"
#include "monitor.h"
#include "interrupt.h"

#include "fdc_detect.h"
#include "fdc_driver.h"


//#define USE_WRITE_TEST


__sfr __at 		0xce 	rDMD;

// porty k mapovani standardni pameti
__sfr __at 		0xe0 	rMAPE0;
__sfr __at 		0xe1 	rMAPE1;
__sfr __at 		0xe2 	rMAPE2;
__sfr __at 		0xe3 	rMAPE3;
__sfr __at 		0xe4 	rMAPE4;
__sfr __at 		0xe5 	rMAPE5;
__sfr __at 		0xe6 	rMAPE6;



void prg_error ( unsigned char err_code ) {
    xprintf ( " ERR = %d", err_code );
    while ( 1 ) {
        rBCOLOR = err_code;
        rBCOLOR = 0x00;
    };
}


void print_result ( char *s ) {
    cur.pos_x = 40;
    MZ800_INVPRNon ();
    prnc ( '[' );
    msg ( s );
    prnc ( ']' ); 
    MZ800_INVPRNoff ();
}


/*
 * Kod provadeny pod IM2
 *
*/
void my_isr ( void ) __naked {

    __asm
    
    push bc
    push af
    push hl
    
    
    ld bc,#0x06cf
    ld a,#0x04
    out (c),a

    call _fdc_im2_service
    
    ld b,#0x00
    
    1$:
    djnz 1$
    
    ld b,#0x06
    xor a
    out (c),a
    
    pop hl    
    pop af
    pop bc
    
    
    ei
    reti
    
    __endasm;
}




void main ( void ) {

#ifdef USE_WRITE_TEST
    char tsttxt[] = "NAZDARRRRR BAZAR!";
    char *c = tsttxt;
    unsigned int addr;
#endif

    FDC_DRIVE_ID drive = FDC_DRIVE0;

    unsigned char unicard_fdc;
    
    unsigned char track = 1;
    unsigned char sector = 1;
    
    FDC_DRIVE_PARAM drive_params = FDC_SECTOR_SIZE_256 | FDC_MEDIUM_SIDES2 | FDC_MEDIUM_NOT_CPL;
    
    void *dma = (void*) 0x6000;
        
    char err;
    
    xputc = prnc;
    xputs = msg;


    rBCOLOR = BLACK;
    
    /* nainstalujeme IM2 interrupt */
    //    interrupt_pioz80_data[3] = 0x97;  // zacatek zatemneni (implicitni nastaveni)
    //* ( &interrupt_pioz80_data + 3 ) = 0xb7;
    interrupt_install_isr ( my_isr );
            
    
    
    
    rPAL = PAL0 | BLACK;
    rPAL = PAL1 | LWHITE;
    rPAL = PAL2 | WHITE;
    rPAL = PAL3 | BLACK;
    
    rWF = PLANE_I | PLANE_III;
    video_cls ();
    rWF = PLANE_I;
    MZ800_INVPRNoff ();
    cur.pos_x = 0;
    cur.pos_y = 0;



    /* mame pripojen radic? */
    xprintf ( "FDC - test:" );

    if ( 0 == fdc_detect () ) {
        print_result ( "OK" );
    } else {
        print_result ( "NOT FOUND" );
        // radic nenalezen, nema smysl dal pokracovat
        while ( 1 ) {};
    };


    /* mame pripojen druhy radic na Unikarte? */
    cur.pos_x = 0;
    cur.pos_y += 2;
    xprintf ( "FDC on Unicard - test:" );

    if ( 0 == fdc_detect_unicard () ) {
        print_result ( "OK" );
        unicard_fdc = 1;
    } else {
        print_result ( "NOT FOUND" );
        unicard_fdc = 0;
    };
    

    /* info o timingu */
    cur.pos_x = 0;
    cur.pos_y += 2;
    
    xprintf ( "FDC timing:" );
    
    cur.pos_x = 40;
    cur.pos_y ++;
        
    xprintf ( "motor = %d ms", _fdc_timing.start_time );

    cur.pos_x = 40;
    cur.pos_y ++;
    
    xprintf ( "track = %d ms", _fdc_timing.track_time );

    cur.pos_x = 40;
    cur.pos_y ++;

    xprintf ( "step  = %d ms", _fdc_timing.step_time );

    cur.pos_x = 40;
    cur.pos_y ++;

    xprintf ( "off   = %d calls", _fdc_timing.motor_predef );




    /* cteni sektoru */
    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Read sectors:" );
    
    cur.pos_x = 0;
    cur.pos_y++;
    xprintf ( "drv=%d,dma=0x%04x,trk=%d,sec=%d,prm=0x%02x:", drive,  (unsigned int) dma, track, sector, drive_params );

    err = fdc_read_sector ( drive, drive_params, track, sector, dma );

    if ( err ) prg_error ( err );
    print_result ( "OK" );


    cur.pos_x = 0;
    cur.pos_y++;

//    xprintf ( "drv=%d,dma=0x%04x,trk=%d,sec=%d,prm=0x%02x:", drive,  ((unsigned int) dma + 0x1000), track, ( sector + 1), drive_params );

//    err = fdc_read_sector ( drive, drive_params, track, (sector + 1), ((unsigned int) dma + 0x1000) );
//    if ( err ) prg_error ( err );    
//    print_result ( "OK" );

    if ( unicard_fdc ) {
        drive = FDC_DRIVE4;
    
        xprintf ( "drv=%d,dma=0x%04x,trk=%d,sec=%d,prm=0x%02x:", drive,  ((unsigned int) dma + 0x1000), track, sector, drive_params );

        err = fdc_read_sector ( drive, drive_params, track, sector, ((unsigned int) dma + 0x1000) );
        if ( err ) prg_error ( err );    
        print_result ( "OK" );
    };


#ifdef USE_WRITE_TEST
    /* zapiseme do pameti vtipnou hlasku */
    addr = (unsigned int) dma;
    
    while ( c[0] != '\0' ) {
        * (char *) addr++ = c[0];
        c++;
    };



    /* zapis sektoru */
    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Write sector:" );


    cur.pos_x = 0;
    cur.pos_y++;

    xprintf ( "drv=%d,dma=0x%04x,trk=%d,sec=%d,prm=0x%02x:", drive, (unsigned int) dma, track, (sector + 1), drive_params );

    err = fdc_write_sector ( drive, drive_params, track, (sector + 1), dma );
    if ( err ) prg_error ( err );
    print_result ( "OK" );
#endif


    /* konec */

/*  k vypnuti dojde pod IM2
    
    fdc_force_motor_off ();
*/    

    cur.pos_x = 0;
    cur.pos_y += 2; 
    xprintf ( "Done." );
    
    
    while ( 1 ) {};
}
